/*****************************************************************************
 * PDMP3 - A MP3 manager for GNOME                                           *
 * Copyright (C) 2000 Scott Guilbeaux <scottg@viptx.net>                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *****************************************************************************/

#include "pdmp3.h"

typedef struct {
   unsigned int byte1;
   unsigned int byte2;
   unsigned int byte3;
   unsigned int byte4;
} frame_header_t;

/*
 * Read in a frame header to parse
 */

static void read_frame_header(FILE *fp, frame_header_t *frame_header)
{
   int c;
   
   c = fgetc(fp);
   
   while(c != EOF)
     {
	if(c == 0xFF)
	  {
	     /* get the next byte */
	     c = fgetc(fp);
	     if((c & 0xE0) == 0xE0)
	       {
		  frame_header->byte1 = 0xFF;
		  frame_header->byte2 = c;
		  frame_header->byte3 = fgetc(fp);
		  frame_header->byte4 = fgetc(fp);
		  break;
	       }
	     else
	       ungetc(c, fp);
	  }
	c = fgetc(fp);
     }
   fclose(fp);
}

/*
 * Get mpeg version from the header
 */

static gint get_mpeg_version(frame_header_t *frame_header)
{
   gint mpeg_version;
   gint retval = -1;
   
   mpeg_version = frame_header->byte2 & 0x18;
   
   if(mpeg_version == 0x18)
     retval = 1;
   else if(mpeg_version == 0x10)
     retval = 2;
   else if(mpeg_version == 0x08)
     retval = -1;
   else if(mpeg_version == 0x00)
     retval = 3;
   
   return retval;
}

/*
 * Get the mpeg layer from the header
 */

static gint get_mpeg_layer(frame_header_t *frame_header)
{
   gint layer;
   gint retval = -1;
   
   layer = frame_header->byte2 & 0x06;
   
   if(layer == 0x02)
     retval = 3;
   else if(layer == 0x06)
     retval = 1;
   else if(layer == 0x04)
     retval = 2;
   else if(layer == 0x00)
     retval = -1;
   
   return retval;
}

/*
 * Check to see if the mp3 is crc protected
 */

static gint is_crc_protected(frame_header_t *frame_header)
{
   gint retval = 0;
   gint crc_bit;
   
   crc_bit = frame_header->byte2 & 0x01;
   
    if(crc_bit == 0x01)
     retval = 1;
   else if(crc_bit == 0x00)
     retval = 0;
   
   return retval;
}

/*
 * Get the bit rate from the header
 */

static gint get_bit_rate(frame_header_t *frame_header, gint layer, gint version)
{
   gint retval = -1;
   gint bit_rate;
   
   bit_rate = frame_header->byte3 & 0xF0;
   
   /* BITRATE 0000 */
   if(bit_rate == 0x00)
     retval = 0;
   
   /* BITRATE 0001 */
   else if(bit_rate == 0x10)
     {
	if(version >= 2)
	  {
	     if((layer == 3) || (layer == 2))
	       retval = 8;
	     else
	       retval = 32;
	  }
	else if(version == 1)
	  retval = 32;
     }
 
   /* BITRATE 0010 */
   else if(bit_rate == 0x20)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 64;
	     else if(layer == 2)
	       retval = 48;
	     else if(layer == 3)
	       retval = 40;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 48;
	     else if((layer == 3) || (layer == 2))
	       retval = 16;
	  }
     }

   /* BITRATE 0011 */
   else if(bit_rate == 0x30)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 96;
	     else if(layer == 2)
	       retval = 56;
	     else if(layer == 3)
	       retval = 48;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 56;
	     else if((layer == 3) || (layer == 2))
	       retval = 24;
	  }
     }
   
   /* BITRATE 0100 */
   else if(bit_rate == 0x40)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 128;
	     else if(layer == 2)
	       retval = 64;
	     else if(layer == 3)
	       retval = 56;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 64;
	     else if((layer == 3) || (layer == 2))
	       retval = 32;
	  }
     }
   
   /* BITRATE 0101 */
   else if(bit_rate == 0x50)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 160;
	     else if(layer == 2)
	       retval = 80;
	     else if(layer == 3)
	       retval = 64;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 80;
	     else if((layer == 3) || (layer == 2))
	       retval = 40;
	  }
     }
   
   /* BITRATE 0110 */
   else if(bit_rate == 0x60)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 192;
	     else if(layer == 2)
	       retval = 96;
	     else if(layer == 3)
	       retval = 80;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 96;
	     else if((layer == 3) || (layer == 2))
	       retval = 48;
	  }
     }
   
   /* BITRATE 0111 */
   else if(bit_rate == 0x70)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 224;
	     else if(layer == 2)
	       retval = 112;
	     else if(layer == 3)
	       retval = 96;
	  }
	else if(version == 2)
	  {
	     if(layer == 1)
	       retval = 112;
	     else if((layer >= 2) || (layer == 3))
	       retval = 56;
	  }
     }
   
   /* BITRATE 1000 */
   else if(bit_rate == 0x80)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 256;
	     else if(layer == 2)
	       retval = 128;
	     else if(layer == 3)
	       retval = 112;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 128;
	     else if((layer == 2) || (layer == 3))
	       retval = 64;
	  }
     }
   
   /* BITRATE 1001 */
   else if(bit_rate == 0x90)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 288;
	     else if(layer == 2)
	       retval = 160;
	     else if(layer == 3)
	       retval = 128;
	  }
	else if(version == 2)
	  {
	     if(layer == 1)
	       retval = 144;
	     else if((layer >= 2) || (layer == 3))
	       retval = 88;
	  }
     }
     
   /* BITRATE 1010 */
   else if(bit_rate == 0xA0)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 320;
	     else if(layer == 2)
	       retval = 192;
	     else if(layer == 3)
	       retval = 160;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 160;
	     else if((layer == 2) || (layer == 3))
	       retval = 96;
	  }
     }
   
   /* BITRATE 1011 */
   else if(bit_rate == 0xB0)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 352;
	     else if(layer == 2)
	       retval = 224;
	     else if(layer == 3)
	       retval = 192;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 176;
	     else if((layer == 2) || (layer == 3))
	       retval = 112;
	  }
     }
   
   /* BITRATE 1100 */
   else if(bit_rate == 0xC0)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 384;
	     else if(layer == 2)
	       retval = 256;
	     else if(layer == 3)
	       retval = 224;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 192;
	     else if((layer == 2) || (layer == 3))
	       retval = 128;
	  }
     }
   
   /* BITRATE 1101 */
   else if(bit_rate == 0xD0)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       bit_rate = 416;
	     else if(layer == 2)
	       bit_rate = 320;
	     else if(layer == 3)
	       bit_rate = 256;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 224;
	     else if((layer == 2) || (layer == 3))
	       retval = 144;
	  }
     }
   
   /* BITRATE 1110 */
   else if(bit_rate == 0xE0)
     {
	if(version == 1)
	  {
	     if(layer == 1)
	       retval = 448;
	     else if(layer == 2)
	       retval = 384;
	     else if(layer == 3)
	       retval = 320;
	  }
	else if(version >= 2)
	  {
	     if(layer == 1)
	       retval = 256;
	     else if((layer == 2) || (layer == 3))
	       retval = 160;
	  }
     }
   
   /* BITRATE 1111 */
   else if(bit_rate == 0xF0)
     retval = -1;
   
   else
     retval = -1;
   
   return retval;
}

/*
 * Get the sampling rate from the header
 */

static gint get_sampling_rate(frame_header_t *frame_header, gint layer, gint version)
{
   gint retval = -1;
   gint sampling_rate;
   
   sampling_rate = frame_header->byte3 & 0x0C;
   
   /* SAMPLING RATE 00 */
   if(sampling_rate == 0x00)
     {
	if(version == 1)
	  retval = 44100;
	else if(version == 2)
	  retval = 22050;
	else if(version == 3)
	  retval = 11025;
     }
   /* SAMPLING RATE 01 */
   else if(sampling_rate == 0x04)
     {
	if(version == 1)
	  retval = 48000;
	else if(version == 2)
	  retval = 24000;
	else if(version == 3)
	  retval = 12000;
     }
   /* SAMPLING RATE 10 */
   else if(sampling_rate == 0x08)
     {
	if(version == 1)
	  retval = 32000;
	else if(version == 2)
	  retval = 16000;
	else if(version == 3)
	  retval = 8000;
     }
   /* SAMPLING RATE 11 */
   else if(sampling_rate == 0x0C)
     retval = -1;
   else
     retval = -1;
   
   return retval;
}

/*
 * Check to see if the padding bit is set
 */

static gint get_padding_bit(frame_header_t *frame_header)
{
   gint retval = 0;
   gint padding_bit;
   
   padding_bit = frame_header->byte3 & 0x02;
   if(padding_bit == 0x02)
     retval = 1;
   else
     retval = 0;
   
   return retval;
}

/*
 * Check to see if the private bit is set
 */

static gint get_private_bit(frame_header_t *frame_header)
{
   gint retval = 0;
   gint private_bit;
   
   private_bit = frame_header->byte3 & 0x01;
   if(private_bit == 0x01)
     retval = 1;
   else
     retval = 0;
   
   return retval;
}

/*
 * Get the channel mode from the header
 */

static gchar *get_channel_mode(frame_header_t *frame_header)
{
   gchar *temp = NULL;
   gint channel_mode;
   
   channel_mode = frame_header->byte4 & 0xC0;
   if(channel_mode == 0x00)
     temp = g_strdup_printf("Stereo");
   else if(channel_mode == 0x40)
     temp = g_strdup_printf("Joint Stereo (Stereo)");
   else if(channel_mode == 0x80)
     temp = g_strdup_printf("Dual Channel (Stereo)");
   else if(channel_mode == 0xC0)
     temp = g_strdup_printf("Single Channel (Mono)");
   
   return temp;
}

/*
 * Get the mode extension from the header
 */

static gchar *get_mode_extension(frame_header_t *frame_header, gint layer)
{
   gchar *temp = NULL;
   gint mode_extension;
   gint tmp;
   
   mode_extension = frame_header->byte4 & 0x30;
   
   tmp = frame_header->byte4 & 0xC0;
   
   /* Check to see if channel mode is joint stereo */
   if(tmp == 0x40)
     {
	if((layer == 1) || (layer == 2))
	  {
	     if(mode_extension == 0x00)
	       temp = g_strdup_printf(N_("Intensity stereo on bands 4 to 31"));
	     else if(mode_extension == 0x10)
	       temp = g_strdup_printf(N_("Intensity stereo on bands 8 to 31"));
	     else if(mode_extension == 0x20)
	       temp = g_strdup_printf(N_("Intensity stereo on bands 12 to 31"));
	     else if(mode_extension == 0x30)
	       temp = g_strdup_printf(N_("Intensity stereo on bands 16 to 31"));
	  }
	else if(layer == 3)
	  {
	     if(mode_extension == 0x00)
	       temp = g_strdup_printf(N_("Intensity stereo off, MS stereo off"));
	     else if(mode_extension == 0x10)
	       temp = g_strdup_printf(N_("Intensity stereo on, MS stereo off"));
	     else if(mode_extension == 0x20)
	       temp = g_strdup_printf(N_("Intensity stereo off, MS stereo on"));
	     else if(mode_extension == 0x30)
	       temp = g_strdup_printf(N_("Intensity stereo on, MS stereo on"));
	  }
     }
   else
     temp = g_strdup_printf(N_("No mode extension"));
   return temp;
}

/*
 * Check to see if the copyright bit is set
 */

static gint get_copyright(frame_header_t *frame_header)
{
   gint retval = 0;
   gint copyright;
   
   copyright = frame_header->byte4 & 0x08;
   if(copyright == 0x00)
     retval = 0;
   else if(copyright == 0x08)
     retval = 1;
   
   return retval;
}

/*
 * Check to see if the original bit is set
 */

static gint get_original(frame_header_t *frame_header)
{
   gint retval = 0;
   gint original;
   
   original = frame_header->byte4 & 0x04;
   if(original == 0x00)
     retval = 0;
   else if(original == 0x04)
     retval = 1;
   
   return retval;
}

/*
 * Get the empahsis from the header
 */

static gchar *get_emphasis(frame_header_t *frame_header)
{
   gchar *temp = NULL;
   gint emphasis;
   
   emphasis = frame_header->byte4 & 0x03;

   if(emphasis == 0x00)
     temp = g_strdup_printf(N_("No Emphasis"));
   else if(emphasis == 0x01)
     temp = g_strdup_printf(N_("50/15 ms"));
   else if(emphasis == 0x02)
     temp = g_strdup_printf(N_("Reserved"));
   else if(emphasis == 0x03)
     temp = g_strdup_printf(N_("CCIT J.17"));
   
   return temp;
}

/*
 * Create our info window and fill it in
 */

void pdmp3_info_window_new(void)
{
   GtkWidget *window;
   GtkWidget *frame;
   GtkWidget *label;
   GtkWidget *table;
   gchar *temp;
   frame_header_t frame_header;
   FILE *fp;
   gint x = 0;
   gint y = 0;
   gint result;
   
   memset(&frame_header, 0, sizeof(frame_header));
   fp = fopen(pdmp3.selected_file, "rb");
   
   /* Read the header and get the diffrent
    * fields */
   read_frame_header(fp, &frame_header);
   
   gdk_window_get_position(pdmp3.main_window->window, &x, &y);
   
   window = gnome_dialog_new(N_("File Info"),
			     GNOME_STOCK_BUTTON_CLOSE,
			     NULL);
   gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
   
   temp = g_strdup_printf(N_("File info for: %s\n"), g_basename(pdmp3.selected_file));
   frame = gtk_frame_new(temp);
   
   table = gtk_table_new(12,2,TRUE);
      
   label = gtk_label_new(N_("MPEG Version: "));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,0,1,0,0,0,0);
   label = gtk_label_new(g_strdup_printf("%d", get_mpeg_version(&frame_header)));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 1,2,0,1,0,0,0,0);
   
   label = gtk_label_new(N_("MPEG Layer: "));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,1,2,0,0,0,0);
   label = gtk_label_new(g_strdup_printf("%d", get_mpeg_layer(&frame_header)));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 1,2,1,2,0,0,0,0);
   
   if(is_crc_protected(&frame_header))
     label = gtk_label_new(N_("CRC Protected"));
   else
     label = gtk_label_new(N_("No CRC Protection"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,2,3,0,0,0,0);
   
   label = gtk_label_new(N_("Bit Rate: "));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,3,4,0,0,0,0);
   label = gtk_label_new(g_strdup_printf("%d", get_bit_rate(&frame_header, 
							    get_mpeg_layer(&frame_header),
							    get_mpeg_version(&frame_header))));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 1,2,3,4,0,0,0,0);
   
   label = gtk_label_new(N_("Sampling Rate: "));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,4,5,0,0,0,0);
   label = gtk_label_new(g_strdup_printf("%d", get_sampling_rate(&frame_header,
								 get_mpeg_layer(&frame_header),
								 get_mpeg_version(&frame_header))));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 1,2,4,5,0,0,0,0);
   
   if(get_padding_bit(&frame_header))
     label = gtk_label_new(N_("Padding Bit Set"));
   else
     label = gtk_label_new(N_("Padding Bit Not Set"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,5,6,0,0,0,0);
   
   if(get_private_bit(&frame_header))
     label = gtk_label_new(N_("Private Bit Set"));
   else
     label = gtk_label_new(N_("Private Bit Not Set"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,6,7,0,0,0,0);
   
   label = gtk_label_new(N_("Channel Mode: "));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,7,8,0,0,0,0);
   label = gtk_label_new(get_channel_mode(N_(&frame_header)));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 1,2,7,8,0,0,0,0);
   
   label = gtk_label_new(N_("Mode Extension: "));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,8,9,0,0,0,0);
   label = gtk_label_new(get_mode_extension(&frame_header, get_mpeg_layer(&frame_header)));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 1,2,8,9,0,0,0,0);
   
   if(get_copyright(&frame_header))
     label = gtk_label_new(N_("Copyrighted"));
   else
     label = gtk_label_new(N_("No Copyright"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label,0,1,9,10,0,0,0,0);
   
   if(get_original(&frame_header))
     label = gtk_label_new(N_("Orignal Media"));
   else
     label = gtk_label_new(N_("Copy of Original"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,10,11,0,0,0,0);
   
   label = gtk_label_new(N_("Emphasis: "));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0,1,11,12,0,0,0,0);
   label = gtk_label_new(get_emphasis(&frame_header));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 1,2,11,12,0,0,0,0);
   
   gtk_container_add(GTK_CONTAINER(frame), table);
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(window)->vbox), frame,
		      TRUE, TRUE, 0);
   
   gtk_widget_show_all(GTK_WIDGET(frame));
   
   for(;;)
     {
	result = gnome_dialog_run(GNOME_DIALOG(window));
	if(result == 0)
	  {
	     gnome_dialog_close(GNOME_DIALOG(window));
	     break;
	  }
	else if(result < 0)
	  break;
     }
}
